UNIT SOCKStrc;

{ GNU PASCAL unit for sockets - version 1.2 }

{ 
Copyright (C) 1997-2003 Clyde Meli <cmeli@cis.um.edu.mt> and
V. Nezval <vnez@cis.um.edu.mt>
of the Department of Computer Information Systems, University of Malta, Malta

SOCKStrc and associated files are free software; you can redistribute and/or
modify them under the terms of the GNU Library Public License, as published
by the Free Software Foundation.

You should have received a copy of the GNU Library Public License (file
COPYING.LIB) along with this package; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

SOCKStrc and its associated files are distributed in the hope that they
will be useful, and that others may learn from them, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Library Public License for more details.


  We developed this program for use by third year informatics students taking ISA 308 (Client Server) - now CIS3080.
  Our students learn PASCAL from the beginning of their course, so GNU PASCAL
  has been an extremely useful compiler for practicing client-server programming.

 This unit has only been tested under Solaris 2.5.1 (sparc), it is a wrapper
 for using the solaris socket C library. The constants declared in the 
 beginning of the unit are correct for Solaris 2.5.1, taken from the C
 system header files.  Your mileage may vary.

 We have decided to submit this sockets unit as a contrib for the GNU PASCAL
 distribution. We will be doing this as soon as possible.

 Please send us your improvements, etc. All comments are appreciated.
 The latest version should be available at

   http://www.cis.um.edu.mt/staff/cmeli/

}

INTERFACE
uses c_types;

CONST { sys/socket.h }
  SOCK_STREAM = 2; { for solaris, this must be 2 }
  SOCK_DGRAM = 1; 
  INADDR_ANY = 0;
  PF_INET = 2;
  AF_INET = 2;  

TYPE

  str8=string[8];
  str255=string[255];
  intptr=^integer;

sockaddrptr=^sockaddr; { sockaddr was taken 
			from /usr/include/sys/socket.h }
sockaddr=RECORD
             sa_family:ushort;
             sa_data:packed array[0..13] of char;
           end;

{ sockaddr_in was taken from /usr/include/netinet/in.h }
in_addrptr=^in_addr;
in_addr=RECORD   
            S_addr:ulong;
        end;
sockaddr_inptr = ^ sockaddr_in;
sockaddr_in=RECORD
      		sin_family:short;
      		sin_port:ushort;
      		sin_addr:in_addr;
      		sin_zero:str8; {PACKED ARRAY[1..8] of char;}
    		end;

{PChar = ^Char;  pointer to array of chars - defined in latest beta }

{ hostent was taken from /usr/include/sys/ }
hostentptr = ^hostent;
hostent = RECORD
             h_name:Cstring; 		{official name of host}
             h_aliases:Cstringptr; 	{ alias list}
             h_addrtype:integer; 	{ host address type }
             h_length:integer; 		{ length of address }
 	     h_addr_list:Cstringptr;	{list of addresses from name           
					 server. If one only it is the
					 first }
          end;

protoentptr = ^protoent;
protoent = RECORD
             p_name:Cstring; 		{official protocol name }
             p_aliases:Cstringptr; 	{ alias list}
             p_proto:integer; 	{ protocol binary number }
          end;

serventptr = ^servent;
servent = RECORD
             s_name:Cstring; 		{official service name}
             s_aliases:Cstringptr; 	{ alias list}
             s_port:integer; 		{ serv. port number - binary }
             s_proto:Cstring;		{ protocol to use }
 	    end;

   pid_t = long;	{for fork() if needed }

   socklen_tptr = ^socklen_t;
   socklen_t = integer;

{=============== CONNECTING Functions ====================}

function socket ( domain:integer;typ:integer;protocol:integer):integer;C;

function connect (s:integer;name:sockaddrptr;namelen:socklen_t{integer}):integer;C;

function bind ( s:integer; name:sockaddrptr;namelen:socklen_t{integer}):integer;C;

function listen ( s: integer; qlen : integer ): integer; C;
{puts the socket into passive mode restricting number of client connections to qlen}

function accept ( s:integer; name:sockaddrptr;addrlenn:intptr):integer;C;

{------------------ C equivalents -------------------}
{ int socket ( int domain, int type, int protocol); }
{ int connect(int s, struct sockaddr *name, int namelen); }
{ int bind(int s, const struct sockaddr *name, int namelen); }
{ int accept(int s, struct sockaddr *name, int *addrlen); }

function getpeername(sockfd:integer; name:sockaddrptr;namelen:socklen_tptr):integer;

{=============== address conversion tools ====================}

function PAS_gethostbyname ( name:str255 ): hostentptr;
        {to convert host name into binary form. name is a Pascal string }
function gethostbyname ( name:Cstring ): hostentptr;C;
	{to convert host name into binary form. name is a C format string}

function PAS_inet_addr ( cp:str255 ): ulong;

function inet_addr (cp:Cstring):ulong;C;
	{to convert host decimal dotted address into binary}

function PAS_inet_network( cp:str255 ):ulong;

function inet_network(cp:Cstring):ulong;C;
	{to convert network decimal dotted address into binary}
{ NB: Solaris doesn't support inet_aton }

function inet_makeaddr (net:integer;lna:integer):in_addr;C;
{to make host address by placing its binary equivalent into 
in_addr. Params are binary addresses of network and of host:
E.g.: sin_addr:=inet_makeaddr(inet_network(pNetwName),inet_addr(pHostName));}

function PAS_inet_ntoa( inad:in_addr ):str255;

function inet_ntoa( inad:in_addr ):CString;C; 
{to convert host address from binary to decimal dotted address }

{------------------ C equivalents -------------------}
{ struct hostent *gethostbyname(const char *name);  --note ignore CONST}
{ unsigned long inet_addr(const char *cp); }
{ unsigned long inet_network(const char *cp); }
{ struct in_addr inet_makeaddr(const int net, const int lna); }
{ char *inet_ntoa(const struct in_addr in); }

{=============== transport protocol conversion tools =============}
function getprotobyname ( name:Cstring ): protoentptr;C;
	{to convert transport string ("TCP" or "UDP) into into binary form}
{------------------ C equivalents -------------------}
{ struct protoent *getprotobyname(const char *name);  --note ignore CONST}

{========= service protocol name conversion tool (for UDP)======}
function getservbyname ( name:Cstring; proto:CString ): serventptr;C;
	{to convert service name into binary serv. prot. number}
{------------------ C equivalents -------------------}
{ struct servent *getservbyname(const char *name,const char *proto);}

{====================== TRANSFER of Data Functions ================}

function send(sockfd:integer;msg:Cstring;len:integer;flags:integer):integer;C;
	{socket number, message, length of message, flags ... just 0 }

function recv(sockfd:integer;buf:Cstring;len:integer;flags:integer):integer;C;
{socket number, buffer, length read,  flags ... just 0 }

{------------------ C equivalents -------------------}
{ int send(int sockfd, const void *msg, int len, int flags); }
{ int recv(int sockfd, void *buf, int len, unsigned int flags); }
{====================== CLOSING Functions ===============}

procedure closesocket (sockfd:integer);C;
{ close a socket - CHANGE from previous versions used close }

function shutdown(sockfd:integer;how:integer): integer;C;


{
 TO BE ADDED
int getsockopt(int s, int level, int optname, char *optval,
          int *optlen);
     int setsockopt(int s, int level, int optname,
          const char *optval, int optlen);

}

{====================== byte and string conversion Functions ================}
function htons(hostshort : ushort) : ushort;C;
{converts from host to network byte order }

function ntohs(netshort : ushort) : ushort;C;
{converts from network byte to host byte order }

function atoi(st: Cstring):integer; C;
{converts an ASCIIZ string into integer}

{====================== MISCELLANEOUS Functions ================}
function fork:pid_t;C;
	{ pid_t fork(void); }

function sleep(seconds: word): word; C;

function getchar:integer;C;

{ NOTE for students learning sockets - a good site is
  http://www.ecst.csuchico.edu/~beej/guide/net  }

{====================== STRING CONVERSION Functions ================}
function StrPas(Src: CString): TString;
{ Convert a "Pascal" string to a "C" string. The converted string is 
both returned and also placed in Dest. Dest is to be allocated space on
the heap before calling this function  }

function StrPCopy(Dest: CString; Src: TString): CString;
{ Convert a "Pascal" string to a "C" string }

IMPLEMENTATION

function StrPas(Src: CString): TString;
{ Convert a "C" string to a "Pascal" string }
var
  S : TString;
begin
  S := '';
  if (Src <> NIL)
  then while ( (Src^ <> chr(0)) AND (length(S) < S.capacity)) do
  begin
    S := S + Src^;
    inc(Word(Src));
  end;
  StrPas := S;
end;
     
function StrPCopy(Dest: CString; Src: TString): CString;
{ Convert a "Pascal" string to a "C" string. The converted string is 
both returned and also placed in Dest. Dest is to be allocated space on
the heap before calling this function  }
var
  c: integer;
  p : CString;
begin
  p := Dest;
  for c:=1 to length(Src) do
  begin
    p^ := Src[c];
    inc(word(p));
  end;
  p^ := chr(0);
  StrPCopy := Dest;
end;

function PAS_gethostbyname ( name:str255 ): hostentptr;
        {to convert host name into binary form. name is a Pascal string }
VAR
  cname:array[0..255] of byte;
Begin
  StrPCopy(CString(@cname),name); { Convert name to an ASCIIZ string }
  PAS_gethostbyname:=gethostbyname(CString(@cname));
End;

function PAS_inet_addr ( cp:str255 ): ulong;
        {to convert host decimal dotted address into binary}
VAR
  temp:array[0..255] of byte;
Begin
  StrPCopy(CString(@temp),cp);
  PAS_inet_addr:=inet_addr(CString(@temp));
End;

function PAS_inet_network( cp:str255 ):ulong;
        {to convert network decimal dotted address into binary}
VAR
  temp:array[0..255] of byte;
Begin
  StrPCopy(CString(@temp),cp);
  PAS_inet_network:=inet_network(CString(@temp));
End;

function PAS_inet_ntoa( inad:in_addr ):str255;
{to convert host address from binary to decimal dotted address }
Begin
  PAS_inet_ntoa:=StrPas(inet_ntoa(inad));
End;



END.

